//
//  GLAtlasBrowserDelegate.h
//  GLAtlasKit
//
//  Created by Jeffrey Early on 1/27/08.
//  Copyright 2008 Early Innovations, LLC. All rights reserved.
//

#import <Cocoa/Cocoa.h>

typedef enum 
{
    kGLShouldAlwaysDisplay, 
    kGLShouldConditionallyDisplay,
    kGLShouldNeverDisplay
} GLShouldDisplayColumnType;

@class GLCity, GLCounty, GLState, GLCountry, GLContinent;
@interface GLAtlasBrowserDelegate : NSObject {
	IBOutlet NSBrowser *browser;

	GLContinent *selectedContinent;
	GLCountry *selectedCountry;
	GLState *selectedState;
	GLCounty *selectedCounty;	
	GLCity *selectedCity;
	
	NSArray *continents;
	NSArray *countries;
	NSArray *states;
	NSArray *counties;
	NSArray *cities;
	
	NSInteger continentColumn;
	NSInteger countryColumn;
	NSInteger stateColumn;
	NSInteger countyColumn;
	NSInteger cityColumn;
	
	NSInteger selectedContinentRow;
	NSInteger selectedCountryRow;
	NSInteger selectedStateRow;
	NSInteger selectedCountyRow;
	NSInteger selectedCityRow;
}

// The delegate needs to be wired up to an NSBrowser (it's the browsers delegate, but also hook it up to the browser).
// The browser gives user access to the Atlas database.
// The "Selected Objects" are the objects that the user currently has selected. Some columns can be turned off,
// so just because a State, for example, isn't selected, doesn't mean the city can't identify one.

/************************************************/
/*		Selected Objects						*/
/************************************************/

#pragma mark -
#pragma mark Selected Objects
#pragma mark

@property(assign) GLContinent *selectedContinent;
@property(assign) GLCountry *selectedCountry;
@property(assign) GLState *selectedState;
@property(assign) GLCounty *selectedCounty;
@property(assign) GLCity *selectedCity;

/************************************************/
/*		Browser								*/
/************************************************/

#pragma mark -
#pragma mark Browser
#pragma mark

@property(assign) NSArray *continents;
@property(assign) NSArray *countries;
@property(assign) NSArray *states;
@property(assign) NSArray *counties;
@property(assign) NSArray *cities;

- (void) reloadContinents;
- (void) reloadCountries;
- (void) reloadStates;
- (void) reloadCounties;
- (void) reloadCities;

@property(readonly) GLShouldDisplayColumnType shouldDisplayContinentColumn;
@property(readonly) GLShouldDisplayColumnType shouldDisplayCountryColumn;
@property(readonly) GLShouldDisplayColumnType shouldDisplayStateColumn;
@property(readonly) GLShouldDisplayColumnType shouldDisplayCountyColumn;
@property(readonly) GLShouldDisplayColumnType shouldDisplayCityColumn;

@property NSInteger continentColumn;
@property NSInteger countryColumn;
@property NSInteger stateColumn;
@property NSInteger countyColumn;
@property NSInteger cityColumn;

@property NSInteger selectedContinentRow;
@property NSInteger selectedCountryRow;
@property NSInteger selectedStateRow;
@property NSInteger selectedCountyRow;
@property NSInteger selectedCityRow;

/************************************************/
/*		CoreData								*/
/************************************************/

#pragma mark -
#pragma mark CoreData
#pragma mark

@property(readonly) NSManagedObjectContext *managedObjectContext;

@end
