//
//  GLAtlasKit.h
//  GLAtlasKit
//
//  Created by Jeffrey Early on 11/25/07.
//  Copyright 2007 Early Innovation. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <GLGeometryKit/GLGeometryKit.h>

#import <GLAtlasKit/GLAtlasBrowserDelegate.h>
#import <GLAtlasKit/GLContinent.h>
#import <GLAtlasKit/GLCountry.h>
#import <GLAtlasKit/GLState.h>
#import <GLAtlasKit/GLCounty.h>
#import <GLAtlasKit/GLCity.h>

@interface GLAtlasKit : NSObject {
	NSMutableArray *timezoneArray;
	
	NSArray *availableReverseGeocoders;
	id selectedReverseGeocoder;
	
    NSPersistentStoreCoordinator *persistentStoreCoordinator;
    NSManagedObjectModel *managedObjectModel;
    NSManagedObjectContext *managedObjectContext;

}

/************************************************/
/*		Initialization							*/
/************************************************/

#pragma mark -
#pragma mark Initialization
#pragma mark

+ (id) sharedAtlasKit;

/************************************************/
/*		Time Zone								*/
/************************************************/

#pragma mark -
#pragma mark Time Zone
#pragma mark

@property(assign) NSMutableArray *timezoneArray;

- (NSTimeZone *) timeZoneForPoint: (GLPoint) point;
- (NSTimeZone *) timeZoneForPointGeometry: (GLPointGeometry *) point;

/************************************************/
/*		Reverse Geocoding						*/
/************************************************/

#pragma mark -
#pragma mark Reverse Geocoding
#pragma mark

@property(readonly) NSArray *availableReverseGeocoders;
@property(assign) id selectedReverseGeocoder;
@property NSUInteger selectedReverseGeocoderIndex;

- (NSDictionary *) descriptionForLatitude: (double) latitude longitude: (double) longitude;
- (void) descriptionForLatitude: (double) latitude longitude: (double) longitude delegate: (id) aDelegate didFinishSelector: (SEL) selector;

// Decodes the dictionary, returning a human-readable string describing the location.
- (NSString *) locationDescriptionFromDictionary: (NSDictionary *) dictionary;

/************************************************/
/*		CoreData								*/
/************************************************/

#pragma mark -
#pragma mark CoreData
#pragma mark

@property(readonly) NSPersistentStoreCoordinator *persistentStoreCoordinator;
@property(readonly) NSManagedObjectModel *managedObjectModel;
@property(readonly) NSManagedObjectContext *managedObjectContext;

@end
