//
//  GLGPSBabelImporter.h
//  GLGPSKit
//
//  Created by Jeffrey Early on 6/15/08.
//  Copyright 2008 Early Innovations, LLC. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <GLGPSKit/GLGPSKit.h>

@interface GLGPSBabelImporter : NSWindowController <GLTrackImportProtocol> {

	NSMutableArray		*gpsTypes;
	NSMutableArray		*inputSources;
	NSUInteger			gpsTypesSelectionIndex;
	NSUInteger			inputSourcesSelectionIndex;
	BOOL				shouldDownloadTracks;
	BOOL				shouldDownloadWaypoints;
	NSString			*fileDestinationPath;
	NSMutableArray		*gpsbabelArguments;
	NSTask				*babelTask;
	NSPipe				*babelOutputPipe;
	BOOL				babelTaskErrorOccurred;
	GLGPXImporter			*gpxImporter;
	
	// GLTrackImportProtocol
	BOOL				trackImportIsInProgress;
	BOOL				trackImportIsIndeterminateProgress;
	NSUInteger			trackImportProgressMin;
	NSUInteger			trackImportProgressMax;
	NSUInteger			trackImportProgressValue;
	NSString			*trackImportProgressInfo;
	
	BOOL				trackImportDidFinish;
	NSMutableSet		*events, *waypoints, *tracks, *routes;
	NSMutableArray		*importSources;
	
    NSManagedObjectContext *managedObjectContext;
	NSUndoManager *undoManager;
}

/************************************************/
/*		Initialization							*/
/************************************************/

#pragma mark -
#pragma mark Initialization
#pragma mark

@property(assign) NSManagedObjectContext *managedObjectContext;
@property(assign) NSUndoManager *undoManager;

// Prompts the user with the necessary information required to download.
// Return YES if the importer is prepared to download with the track import protocol.
- (BOOL) prepareToDownload;

/************************************************/
/*		User Selection							*/
/************************************************/

#pragma mark -
#pragma mark User Selection
#pragma mark

@property(assign) NSMutableArray *gpsTypes;
@property(assign) NSMutableArray *inputSources;
@property NSUInteger gpsTypesSelectionIndex;
@property NSUInteger inputSourcesSelectionIndex;
@property BOOL shouldDownloadTracks;
@property BOOL shouldDownloadWaypoints;
@property(copy) NSString *fileDestinationPath;
@property(assign) NSMutableArray *gpsbabelArguments;
@property(assign) NSTask *babelTask;
@property(assign) NSPipe *babelOutputPipe;
@property BOOL babelTaskErrorOccurred;
@property(assign) GLGPXImporter *gpxImporter;

- (IBAction) download: (id) sender;
- (IBAction) cancel: (id) sender;

// Private method for refreshing the available serial ports
- (void) refreshInputSources;


- (void) beginGPXFileImport;

/************************************************/
/*		GLTrackImportProtocol						*/
/************************************************/

#pragma mark -
#pragma mark GLTrackImportProtocol
#pragma mark

/************************************************/
/*		Starting/Stopping						*/
/************************************************/

- (void) beginTrackImport;
- (void) abortTrackImport;

/************************************************/
/*		Progress								*/
/************************************************/

@property BOOL trackImportIsInProgress;
@property BOOL trackImportIsIndeterminateProgress;
@property NSUInteger trackImportProgressMin;
@property NSUInteger trackImportProgressMax;
@property NSUInteger trackImportProgressValue;
@property(copy) NSString *trackImportProgressInfo;

/************************************************/
/*		Finish									*/
/************************************************/

@property BOOL trackImportDidFinish;
@property(assign) NSMutableSet *events, *waypoints, *tracks, *routes;
@property(assign) NSMutableArray *importSources;


@end
