//
//  GLTrack.h
//  Trips
//
//  Created by Jeffrey Early on 9/30/07.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <GLGeometryKit/GLGeometryKit.h>
#import <GLAtlasKit/GLAtlasKit.h>

@class GLTrackSegment, GLEvent;
@interface GLTrack : NSManagedObject <GLSpatialObject> {
	id objectOwner;
	GLGeometry *geometry;
	GLStyle *style;
}

/************************************************/
/*		Basic Attributes						*/
/************************************************/

#pragma mark -
#pragma mark Basic Attributes
#pragma mark

// If a cadence sensor was present, this reports the cadence over the entire track duration. (RPM)
@property(copy) NSNumber * cadence;

// If a heart rate monitor was present, this reports an estimate of the total calories burned. (kCal)
@property(copy) NSNumber * calories;

// Total distance. (meters)
@property(copy) NSDecimalNumber * distance;

// Time interval between the start time (the first point) and the last point. (seconds)
@property(copy) NSDecimalNumber * duration;

// "GPS comment for track." -- GPX1.1 specification --
@property(copy) NSString * gpsComment;

// Users chosen name for the track.
@property(copy) NSString * name;

// User notes. To be used as "desc" for the GPX1.1 specification
@property(copy) NSString * notes;

// "Type (classification) of track." -- GPX1.1 specification -- (Note: stored in GLTrackAdditions table).
@property(copy) NSString * type;

/************************************************/
/*		Date & Time Attributes					*/
/************************************************/

#pragma mark -
#pragma mark Date & Time Attributes
#pragma mark

// Generally equivalent to the time on the first track point.
@property(copy) NSDate * startDate;

// A time zone localized form of the startDateTime
//@property(copy) NSCalendarDate * localStartDate;

// Generally equivalent to the time on the first track point.
//@property(copy) NSDate * endDate;

// A time zone localized form of the startDateTime
//@property(copy) NSCalendarDate * localEndDate;

/************************************************/
/*		Location Metadata						*/
/************************************************/

#pragma mark -
#pragma mark Location Metadata
#pragma mark

// Any of these may be 'nil' depending on a successful query to the AtlasKit
@property(retain) GLCity * city;
@property(retain) GLContinent * continent;
@property(retain) GLCountry * country;
@property(retain) GLCounty * county;
@property(retain) GLState * state;

/************************************************/
/*		Track Segments							*/
/************************************************/

#pragma mark -
#pragma mark Track Segments
#pragma mark

@property(assign) NSArray *segments;
@property(readonly) NSUInteger countOfSegments;
- (GLTrackSegment *) objectInSegmentsAtIndex: (NSUInteger) index;
- (void) insertObject: (GLTrackSegment *) segment inSegmentsAtIndex: (NSUInteger) index;
- (void) removeObjectFromSegmentsAtIndex: (NSUInteger) index;
- (void) addSegment: (GLTrackSegment *) segment;
- (void) removeSegment: (GLTrackSegment *) segment;

/************************************************/
/*		Events									*/
/************************************************/

#pragma mark -
#pragma mark Events
#pragma mark

// Each track belongs to exactly one event
@property(retain) GLEvent * event;

// The order in which this this element was imported into its event.
@property NSInteger order;

// Tracks may belong to zero or more groups
@property(retain) NSSet* manualGroups;

// Is this even necessary? It should be entirely encapsulated in the geometry, no?
@property(readonly) GLRect bounds;

/************************************************/
/*		GLSpatialObject Protocol				*/
/************************************************/

#pragma mark -
#pragma mark GLSpatialObject Protocol
#pragma mark

@property(assign) id objectOwner;
@property(readonly) GLGeometry *geometry;
@property(assign) GLStyle *style;

@end

