//
//  GLTrackImporter.h
//  GLGPSKit
//
//  Created by Jeffrey Early on 11/26/07.
//  Copyright 2007 Early Innovation. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <GLGPSKit/GLTrackImportProtocol.h>

@class GLGPS;
@interface GLTrackImporter : NSWindowController {

	id <GLTrackImportProtocol> importObject;
	id delegate;
	
	BOOL shouldDisplayProgress;
	
	NSArray *validFileTypes;
	NSPredicate *validFileTypePredicate;

    NSManagedObjectContext *managedObjectContext;
	NSMutableArray *importSources;
}

// Events: GPX files are just composed of a list of tracks, with no higher level grouping -- therefore, a GPX
// file does not naturally support the concept of tracks. In this case, perhaps the file itself is the closest
// thing there is to an event.
// TCX files, however, are composed of Activities, each of which does correspond to an Event. In this case the
// file as a component in the organizational structure is irrelevant.

/************************************************/
/*		Import									*/
/************************************************/

#pragma mark -
#pragma mark Import
#pragma mark

// By default this is set to YES and a model import progress dialog will appear.
// Alternatively, this can be set to NO and the importObject can be observed directly.
// !!Not yet implemented.
@property BOOL shouldDisplayProgress;

// A list of the valid file types as strings.
@property(readonly) NSArray *validFileTypes;

// A predicate created using the validFileTypes that allows for quick filtering.
@property(readonly) NSPredicate *validFileTypePredicate;

// Initialize this object, then call one of the three import schemes.
// A progress dialog will display until the import is complete, then the function
// will return.
- (void) importFromFiles: (NSArray *) fileArray;
- (void) importFromGPS: (GLGPS *) gpsDevice;
- (void) importFromGPSBabel;

@property(readonly) BOOL trackImportCompletedSuccessfully;

// After complete import, these three  arrays will contain the imported objects.
@property(readonly) NSSet *events, *waypoints, *tracks, *routes;

// After a complete import *from gpx files* this may contain an array of GLGPXFile objects.
@property(assign) NSMutableArray *importSources;

// The import object adheres to the GLTrackImportProtocol and can be observed accordingly.
@property(assign) id <GLTrackImportProtocol> importObject;

// calls "trackImporterDidFinish:" to the delegate trackImporterDidAddNewImportSourceWithID: 
@property(assign) id delegate;

/************************************************/
/*		CoreData								*/
/************************************************/

#pragma mark -
#pragma mark CoreData
#pragma mark

@property(assign) NSManagedObjectContext *managedObjectContext;

/************************************************/
/*		Private									*/
/************************************************/

#pragma mark -
#pragma mark Private
#pragma mark

- (IBAction) abortImport: (id) sender;
- (void) beginImport;

@end

/************************************************/
/*		Delegate Methods						*/
/************************************************/

#pragma mark -
#pragma mark Delegate Methods
#pragma mark

@interface NSObject (GLTrackImporterDelegate)

- (void) trackImporterDidFinish: (GLTrackImporter *) trackImporter;

@end