//
//  GLTrackManager.h
//  GLGPSKit
//
//  Created by Jeffrey Early on 7/13/08.
//  Copyright 2008 Early Innovations, LLC. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <GLGPSKit/GLGPS.h>
#import <GLGPSKit/GLGeoPoint.h>
#import <GLGPSKit/GLImportSource.h>
#import <GLGPSKit/GLTrackImporter.h>

NSString *GLTrackManagerDidAddNewImportSource;

@interface GLTrackManager : NSObject {
	NSPersistentStoreCoordinator *persistentStoreCoordinator;
	NSManagedObjectContext *managedObjectContext;
	NSArray *validFileTypes;
	NSPredicate *validFileTypePredicate;
}

/************************************************/
/*		Intialization							*/
/************************************************/

#pragma mark -
#pragma mark Intialization
#pragma mark

// Returns a singleton instance of the trackManager with access to the central SQL store.
+ (GLTrackManager *) sharedTrackManager;

// Returns a *new* instance of the trackManager, by default with access to the central SQL store,
// although you can override this by assigning a new persistentStoreCoordinator before the
// NSManagedObjectContext is generated.
+ (GLTrackManager *) trackManager;

// Access to the central SQL database for the trackManager.
+ (NSPersistentStoreCoordinator *) centralPersistentStoreCoordinator;

// An in memory store.
+ (NSPersistentStoreCoordinator *) inMemoryPersistentStoreCoordinator;

// By default this is set to the centralPersistentStoreCoordinator, but you can override this as
// long as you do this before requesting (and therefore initializing) the managedObjectContext.
@property(assign) NSPersistentStoreCoordinator *persistentStoreCoordinator;

// Automatically generated on demand when needed, but can be overridden if necessary.
@property(assign) NSManagedObjectContext *managedObjectContext;

// save the managedObjectContext 
- (void) save;

/********************************************************/
/*		Retrieving Groups, Events, Tracks & Waypoints	*/
/********************************************************/

#pragma mark -
#pragma mark Retrieving Groups, Events, Tracks & Waypoints
#pragma mark

// This method isn't very thorough, but it does get the worst of it, eliminating the majority
// of the faults necessary to delete the entire graph in reasonable time.
- (NSArray *) batchFaultGraphFromImportSource: (GLImportSource *) importSource;

//@property(readonly) NSArray *groups;
//@property(readonly) NSArray *events;
//@property(readonly) NSSet *tracks;
//@property(readonly) NSSet *waypoints;

/************************************************/
/*		Importing New Tracks & Waypoints		*/
/************************************************/

#pragma mark -
#pragma mark Importing New Tracks & Waypoints
#pragma mark

// A list of the valid file types as strings.
@property(readonly) NSArray *validFileTypes;

// A predicate created using the validFileTypes that allows for quick filtering.
@property(readonly) NSPredicate *validFileTypePredicate;

// This method is actually a quick and dirty look at file types and returns YES if one or more files are valid file types.
- (BOOL) validateFilesAtPaths: (NSArray *) paths;

// A track importer object ready to import into this manager's context, and this manager as delegate.
// The root object imported is one (or more) events and can be accessed
// with the function -(NSArray *) lastImportedEvents. The context is NOT saved.
- (GLTrackImporter *) trackImporter;

// Contains the most recent imported objects.
@property(readonly) NSArray *lastImportedEvents;

/************************************************/
/*		GLTrack Points							*/
/************************************************/

#pragma mark -
#pragma mark GLTrack Points
#pragma mark
//
//// returns the nearest point *strictly* preceding in time.
//- (GLGeoPoint *) pointPrecedingTime: (NSDate *) time;
//
//// returns the nearest point following *or equal to* in time.
//- (GLGeoPoint *) pointFollowingTime: (NSDate *) time;

@end
