//
//  GLXMLTrackImporter.h
//  GLGPSKit
//
//  Created by Jeffrey Early on 8/6/08.
//  Copyright 2008 Early Innovations, LLC. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <GLGPSKit/GLTrackImportProtocol.h>
#import <GLGPSKit/GLEvent.h>
#import <GLGPSKit/GLTrack.h>
#import <GLGPSKit/GLTrackSegment.h>
#import <GLGPSKit/GLTrackPoint.h>
#import <GLGPSKit/GLWaypoint.h>

@interface GLXMLTrackImporter : NSObject <GLTrackImportProtocol> {
	NSMutableDictionary *fileLengths;
	NSURL				*currentFile;
	NSMutableArray		*unimportedSources;
	NSUInteger			previousTrackImportProgressValue;
	NSInteger			objectImportOrder;
	NSInteger			eventImportOrder;
	id					delegate;
	
	NSXMLParser			*xmlParser;
	
	NSMutableArray		*importSources;
	NSMutableSet		*events;
	NSMutableSet		*waypoints;
	NSMutableSet		*tracks;
	NSMutableSet		*routes;
	
	NSString			*currentElement;
	NSMutableString		*currentStringValue;
	
	GLEvent				*currentEvent;
	NSMutableSet		*currentTracks;
	GLTrack				*currentTrack;
	NSMutableArray		*currentSegments;
	GLTrackSegment		*currentSegment;
	NSMutableArray		*currentTrackPoints;
	GLTrackPoint		*currentTrackPoint;
	
	GLWaypoint			*currentWaypoint;
	
	BOOL				shouldAbortParsing;
	NSUInteger			currentFileIndex;
	
	// GLTrackImportProtocol
	BOOL				trackImportIsInProgress;
	BOOL				trackImportIsIndeterminateProgress;
	NSUInteger			trackImportProgressMin;
	NSUInteger			trackImportProgressMax;
	NSUInteger			trackImportProgressValue;
	NSString			*trackImportProgressInfo;
	BOOL				trackImportDidFinish;
	
    NSManagedObjectContext *managedObjectContext;
	NSManagedObjectContext *destinationManagedObjectContext;
	NSMutableArray			*contexts;
}

/************************************************/
/*		Initialization							*/
/************************************************/

#pragma mark -
#pragma mark Initialization
#pragma mark

- (id) initWithManagedObjectContext: (NSManagedObjectContext *) context;
- (void) setXMLFiles: (NSArray *) fileArray;
@property(assign) id delegate;

/************************************************/
/*		GLTrackImportProtocol						*/
/************************************************/

#pragma mark -
#pragma mark GLTrackImportProtocol
#pragma mark

/************************************************/
/*		Starting/Stopping						*/
/************************************************/

- (void) beginTrackImport;
- (void) abortTrackImport;

/************************************************/
/*		Progress								*/
/************************************************/

@property BOOL trackImportIsInProgress;
@property BOOL trackImportIsIndeterminateProgress;
@property NSUInteger trackImportProgressMin;
@property NSUInteger trackImportProgressMax;
@property NSUInteger trackImportProgressValue;
@property(copy) NSString *trackImportProgressInfo;

/************************************************/
/*		Finish									*/
/************************************************/

@property BOOL trackImportDidFinish;
@property(assign) NSMutableSet *events, *waypoints, *tracks, *routes;
@property(assign) NSMutableArray *importSources;

/************************************************/
/*		Internal								*/
/************************************************/

#pragma mark -
#pragma mark Internal
#pragma mark

@property(assign) NSMutableArray *unimportedSources;
@property(assign) NSManagedObjectContext *managedObjectContext;
@property(assign) NSManagedObjectContext *destinationManagedObjectContext;

@property NSInteger objectImportOrder, eventImportOrder;

- (void) importNextFile;
- (NSUInteger) numberOfLinesInFile: (NSURL *) fileURL;
- (NSUInteger) numberOfCharactersInFile: (NSURL *) fileURL;

@property(assign) NSMutableDictionary *fileLengths;
@property(copy) NSURL *currentFile;
@property NSUInteger previousTrackImportProgressValue;

@property(assign) NSXMLParser* xmlParser;
@property BOOL shouldAbortParsing;
@property NSUInteger currentFileIndex;

@property(copy) NSString *currentElement;
@property(assign) NSMutableString *currentStringValue;

@property(assign) GLEvent* currentEvent;
@property(assign) NSMutableSet* currentTracks;
@property(assign) GLTrack* currentTrack;
@property(assign) NSMutableArray* currentSegments;
@property(assign) GLTrackSegment* currentSegment;
@property(assign) NSMutableArray* currentTrackPoints;
@property(assign) GLTrackPoint* currentTrackPoint;

@property(assign) GLWaypoint* currentWaypoint;


@end
