//
//  GLAffineTransform.h
//  GLGeometry
//
//  Created by Jeffrey Early on 7/1/06.
//  Copyright 2006, Early Innovation. All rights reserved.
//

#import <Foundation/NSObject.h>
#import <GLGeometryKit/GLGeometry.h>

typedef struct _GLAffineTransformStruct {
    double m11, m12, m21, m22;
    double tX, tY;
} GLAffineTransformStruct;

@interface GLAffineTransform : NSObject <NSCopying, NSCoding>
{
    @public
	GLAffineTransformStruct matrix;
}

// Creation
+ (GLAffineTransform *)transform;

// Initialization
- (id)initWithTransform:(GLAffineTransform *)transform;

// Translating
- (void)translateXBy:(double)deltaX yBy:(double)deltaY;

// Rotating
- (void)rotateByDegrees:(double)angle;
- (void)rotateByRadians:(double)angle;

// Scaling
- (void)scaleBy:(double)scale;
- (void)scaleXBy:(double)scaleX yBy:(double)scaleY;

// Inverting
- (void)invert;

// Transforming with transform
- (void)appendTransform:(GLAffineTransform *)transform;
- (void)prependTransform:(GLAffineTransform *)transform;

// Transforming points and sizes
- (GLPoint)transformPoint:(GLPoint)aPoint;
- (GLSize)transformSize:(GLSize)aSize;
- (GLRect)transformRect:(GLRect)aRect;

// Transform Struct
- (GLAffineTransformStruct)transformStruct;
- (NSAffineTransformStruct)singlePrecisionTransformStruct;
- (void)setTransformStruct:(GLAffineTransformStruct)transformStruct;

- (NSAffineTransform *) nsTransform;

// private stuff, extra
- (void) setMatrix: (const double[6])replace;
- (void) getMatrix: (double[6])replace;

@end
