//
//  GLBezierPathTextExtensions.h
//  GLMapKit
//
//  Created by Jeffrey Early on 4/17/07.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <GLGeometryKit/GLGeometryKit.h>

@interface NSBezierPath (BezierPathGeometryExtensions)

/************************************************/
/*		Symbols									*/
/************************************************/

#pragma mark -
#pragma mark Symbols
#pragma mark

+ (NSBezierPath *) bezierPathWithStarAtPoint: (NSPoint) point withRadius: (float) radius;

/************************************************/
/*		Primary Text Methods					*/
/************************************************/

#pragma mark -
#pragma mark Primary Text Methods
#pragma mark

+ (NSBezierPath *) bezierPathWithText: (NSString *) text forPoint: (GLPoint) point withSymbolizer: (GLTextSymbolizer *) textSymbolizer atScale: (double) scale;
+ (NSArray *) bezierPathsWithText: (NSString *) text forLine: (GLLineString *) lineString withSymbolizer: (GLTextSymbolizer *) textSymbolizer atScale: (double) scale;

+ (NSBezierPath *) pathFromLayoutManager: (NSLayoutManager *) layoutManager font: (NSFont *) font textContainer: (NSTextContainer *) textContainer alongPath: (GLLineString *) lineString
	fromPoint: (int) startIPoint exceeded: (double) startExceeded shift: (double) shift reverseLoop: (BOOL) reverseLoop reverseText: (BOOL) reverseText getEndIPoint: (int *) finalIPoint getEndExceeded: (double *) endexceeded getEndPoint: (GLPoint *) theEnd;
+ (NSArray *) bezierPathsWithText: (NSString *) text curvedAlongPath2: (GLLineString *) lineString withSymbolizer: (GLTextSymbolizer *) textSymbolizer atScale: (double) scale;

+ (NSBezierPath *) pathFromLayoutManager: (NSLayoutManager *) layoutManager font: (NSFont *) font textContainer: (NSTextContainer *) textContainer alongPath: (GLLineString *) lineString
	fromPoint: (int) startIPoint startPoint: (GLPoint) startPoint shift: (double) shift reverseLoop: (BOOL) reverseLoop reverseText: (BOOL) reverseText getEndIPoint: (int *) finalIPoint getEndPoint: (GLPoint *) theEnd success: (BOOL *) success;
+ (NSArray *) bezierPathsWithText: (NSString *) text curvedAlongPath3: (GLLineString *) lineString withSymbolizer: (GLTextSymbolizer *) textSymbolizer atScale: (double) scale;
/************************************************/
/*		Secondary Text Methods					*/
/************************************************/

#pragma mark -
#pragma mark Secondary Text Methods
#pragma mark

// Returns the bezier path object of the text located at the given point using the anchorType to determine
// exactly where in the text's bounding box the point is located.
+ (NSBezierPath *) bezierPathWithText: (NSString *) text font: (NSFont *) font atPoint: (NSPoint) point withAnchor: (GLTextSymbolizerAnchorPositionType) anchorType rotation: (float) rotation;

// Returns the bezier path object of the text. The text returned will attempt to follow lineString, but does
// incorporte a smoothing algorithm that allows for small deviations from the lineString.
+ (NSArray *) bezierPathsWithText: (NSString *) text curvedAlongPath: (GLLineString *) lineString withSymbolizer: (GLTextSymbolizer *) textSymbolizer atScale: (double) scale;

// Returns the bezier path object of the text. The text will be parallel to given portions of the lineString.
+ (NSArray *) bezierPathsWithText: (NSString *) text parallelToPath: (GLLineString *) lineString withSymbolizer: (GLTextSymbolizer *) textSymbolizer atScale: (double) scale;

// Needs to be fixed still
+ (NSBezierPath *) bezierPathWithText: (NSString *) text font: (NSFont *) font inMultiPolygon: (GLMultiPolygon *) multiPolygon repeatSpacing: (double) spacing;

/************************************************/
/*		Supporting Functions					*/
/************************************************/

#pragma mark -
#pragma mark Supporting Functions
#pragma mark

// attempts to find the TRUE width of the text
+ (float) textWidthFromManager: (NSLayoutManager *) layoutManager font: (NSFont *) font textContainer: (NSTextContainer *) textContainer;

// Given a point, this function returns a new point a distance and angle (in degrees) away.
GLPoint displaceOffsetWithAngle( GLPoint originalPoint, double distance, double angle);

// Given a ray defined by point and angle, find the positive max and negatie max (min) distance from the ray
// to the lineString between start and end point.
void getMinMaxDistanceFromLine( GLLineString *lineString, int startIPoint, int endIPoint, GLPoint point, double angle, double *min, double *max);

// Returns the height of the upper case letter in the given font. The letter
// "A" is used as the base definition.
+ (double) upperCaseLetterHeightOfFont: (NSFont *) font;

// Returns the height of the lower case letter in the given font. The letter
// "a" is used as the base definition.
+ (double) lowerCaseLetterHeightOfFont: (NSFont *) font;

// Finds (one) point on the ray formed by nextPoint-lastPoint that is a fixed distance from startPoint
// Returns NO if none exists.
BOOL pointOnRayWithDistance(GLPoint startPoint, GLPoint *endPoint, GLPoint lastPoint, GLPoint nextPoint, double distance);

// Estimates the start of the first glyph.
BOOL firstGlyphStartPoint( GLPoint *startPoint, GLPoint *endPoint, GLPoint *startPoint2, GLPoint *endPoint2, int *nextIPoint, GLSize glyphSize, double shift, GLLineString *lineString, int iPoint, GLPoint previousEndPoint, BOOL reverseLoop );

// lays out the next glyph by hinging on either the top or bottom of the glyph, and demanding that the end point (at the proper shift)
// land on an interpolated point in lineString
BOOL layoutNextGlyph( GLPoint *startPoint, GLPoint *endPoint, int *nextIPoint, GLSize glyphSize, double shift, GLLineString *lineString, int iPoint, BOOL isFirstGlyph, double previousAngle, GLPoint previousEndPoint, BOOL reverseLoop );

// lays out the next glyph, hinging on either the top or the bottom as determined by the forced angle. The end point might not land
// on an interpolated point in lineString
void layoutNextGlyphWithForcedAngle( GLPoint *startPoint, GLPoint *endPoint, int *nextIPoint, GLSize glyphSize, double shift, GLLineString *lineString, int iPoint, double previousAngle, double forcedAngle, GLPoint previousEndPoint, BOOL reverseLoop );

// This forces the iPoint to advance, despite claims that we are far enough along to see if the distance actually decreases.
// This situation can happen with the smoothing. pointOfInterest is the point that we're trying to find a close point on the line to.
int findPointAtDistance( int iPoint, GLLineString *lineString, GLPoint previousPoint, double targetdistance, BOOL reverseLoop );

// This forces the iPoint to advance, searching over 2.0*targetdistance to find the point closest to the targetdistance away
// from the pointOfInterest. It returns targetPoint on that ray, the appropriate distance.
int findLocalMinimum( int iPoint, GLLineString *lineString, GLPoint pointOfInterest, GLPoint *targetPoint, double targetdistance, BOOL reverseLoop);

// Attempts to layout the text along lineString between the two given points
+ (NSBezierPath *) pathFromLayoutManager: (NSLayoutManager *) layoutManager font: (NSFont *) font textContainer: (NSTextContainer *) textContainer alongPath: (GLLineString *) lineString fromPoint: (int) startIPoint toPoint: (int) endIPoint exceededStart: (double) startExceed end: (double) endExceed shift: (double) shift lowerCaseLetterHeight: (double) lowerCaseLetterHeight;

/************************************************/
/*		Geometry Functions						*/
/************************************************/

#pragma mark -
#pragma mark Geometry Functions
#pragma mark

// Given any line string, this will return a new line string with spacing between points equal to outputSpacing
// and will be smoothed on the scale of smoothingScale
GLLineStringPtr smoothLineStringWithBiHarmonic( GLLineStringPtr lineString, double outputSpacing, double smoothingScale );

GLLineString *offsetLineString( GLLineString *lineString, double offset, double minspacing );
BOOL doLineSegmentsCross(GLPoint a1, GLPoint a2, GLPoint b1, GLPoint b2);
GLLineStringPtr pruneLineStringForSpacing( GLLineString *lineString, double minspacing );
double distanceFromLine( GLPoint point, GLLineString *lineString );

int centroid( GLLinearRing *ring, GLPoint *centroid, double *area );
double widthAtPoint( GLLinearRing *ring, GLPoint point );
double signedArea( GLLinearRing *ring );
GLPoint centerOfMass( GLLinearRing *ring );
double area2( GLPoint a, GLPoint b, GLPoint c );
GLPoint Centroid3( GLPoint p1, GLPoint p2, GLPoint p3 );
int circleCircleIntersection( GLPoint center0, double r0, GLPoint center1, double r1, GLPoint *intersection0, GLPoint *intersection1);
@end
