//
//  GLClippingRegion.h
//  GLGeometry
//
//  Created by Jeffrey Early on 2/18/08.
//  Copyright 2008 Early Innovations, LLC. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <GLGeometryKit/GLGeometryTypes.h>
#import <GLGeometryKit/GLGeometryAdditions.h>
#import <GLGeometryKit/GLDrawContext.h>

@class GLAffineTransform;
@interface GLClippingRegion : NSObject {
	GLAffineTransform *transform;
	GLRect clippingRect;
	GLPoint bounds[4];
	GLProjectiveLine edges[4];
	double minSpacing;
}

// pulls the approproriate properties from the draw context
- (GLClippingRegion *) initWithDrawContext: (NSObject <GLDrawContext> *) drawContext;

// After the geometry has been clipped, this is the transformation that should be applied.
@property(assign) GLAffineTransform *transform;

// The clipping rectangle to which the geometry should be clipped.
@property GLRect clippingRect;

// The minimum allowable distance between points.
@property double minSpacing;

// Alternate way of writing the clippingRect (lower-left, lower-right, upper-right, upper-left)
- (GLPoint *) bounds;

// The edges, as lines, in homogenous coordinates.
- (GLProjectiveLine *) edges;

@end
