/*
 *  GLDrawContext.h
 *  GLGeometry
 *
 *  Created by Jeffrey Early on 2/20/08.
 *  Copyright 2008 Early Innovations, LLC. All rights reserved.
 *
 */

#import <GLGeometryKit/GLGeometryTypes.h>
#import <GLGeometryKit/GLGeometryAdditions.h>

// If a view (or otherwise) wants to draw spatialObjects, it needs to conform to this protocol.
// You simply need to call: [spatialObject drawInContext: drawContext] and the spatial object
// will draw its geometry, with the its style.

@class GLAffineTransform, GLMapProjection, GLClippingRegion;
@protocol GLDrawContext

// stores an object representing the projection employed by the draw context, the data source coordinates.
// The geometry will use this object as a weak key to cache a projected version geometry.
@property(readonly) GLMapProjection *mapProjection;

// used for scaling the fonts and geometry widths. Scale factor of 1.0 assumes 72 dpi.
@property(readonly) double scaleFactor;

// The currently displayed resolution of the DrawContext in projected units per pixel.
// Used to remove extraneous points in an optimized path, same as projectedResolution
@property(readonly) double projectedUnitsPerPixel;

// The current display bounds of the DrawContext, in projected coordinates.
@property(readonly) GLRect projectedBounds;

// Transform from projected (data source) coordinates to pixel coordinates
@property(readonly) GLAffineTransform *projectedToPixelTransform;

// The is a more detailed version of the above three properties and includes some precomputed properties useful for 
// speedy clipping.
@property(readonly) GLClippingRegion *clippingRegion;

// Should return yes if the compositer will be drawing with the same geometry objects multiple times
@property(readonly) BOOL shouldCacheSpatialObjectGeometry;

// Should return yes if the compositer is actively scaling. This *may* cause quick and dirty representations
// of complex geometry to be rendered in order to improve rendering times.
@property(readonly) BOOL inLiveScaling;

@end

