//
//  GLGeometryAdditions.h
//  GLGeometry
//
//  Created by Jeffrey Early on 7/1/06.
//  Copyright 2006 Early Innovation. All rights reserved.
//

// Based on NSGeometry (with a few additional functions), taken from OpenStep codebase.

#import <GLGeometryKit/GLGeometry.h>
@class NSString;

static const GLPoint GLZeroPoint = {0.0, 0.0, 0};  /* Zero point. */
static const GLSize GLZeroSize = {0.0, 0.0};    /* Zero size. */
static const GLRect GLZeroRect = {{0.0, 0.0, 0}, {0.0, 0.0}};    /* Zero rectangle. */

static inline GLPoint GLMakePoint(double x, double y) {
    GLPoint p;
    p.x = x;
    p.y = y;
	p.zone = 0;
    return p;
}

static inline GLSize GLMakeSize(double w, double h) {
    GLSize s;
    s.width = w;
    s.height = h;
    return s;
}

static inline GLRect GLMakeRect(double x, double y, double w, double h) {
    GLRect r;
    r.origin.x = x;
    r.origin.y = y;
	r.origin.zone = 0;
    r.size.width = w;
    r.size.height = h;
    return r;
}

static inline double GLMaxX(GLRect aRect) {
    return (aRect.origin.x + aRect.size.width);
}

static inline double GLMaxY(GLRect aRect) {
    return (aRect.origin.y + aRect.size.height);
}

static inline double GLMidX(GLRect aRect) {
    return (aRect.origin.x + aRect.size.width / 2.0);
}

static inline double GLMidY(GLRect aRect) {
    return (aRect.origin.y + aRect.size.height / 2.0);
}

static inline double GLMinX(GLRect aRect) {
    return (aRect.origin.x);
}

static inline double GLMinY(GLRect aRect) {
    return (aRect.origin.y);
}

static inline double GLWidth(GLRect aRect) {
    return (aRect.size.width);
}

static inline double GLHeight(GLRect aRect) {
    return (aRect.size.height);
}

static inline double GLDistance( GLPoint a, GLPoint b ) {
	return sqrt( (a.x - b.x)*(a.x - b.x) + (a.y - b.y)*(a.y - b.y) );
}

#pragma mark -
#pragma mark Comparison
#pragma mark

BOOL GLEqualPoints(GLPoint aPoint, GLPoint bPoint);
BOOL GLEqualSizes(GLSize aSize, GLSize bSize);
BOOL GLEqualRects(GLRect aRect, GLRect bRect);
BOOL GLIsEmptyRect(GLRect aRect);

#pragma mark -
#pragma mark Manipulation
#pragma mark

GLRect GLInsetRect(GLRect aRect, double dX, double dY);
GLRect GLIntegralRect(GLRect aRect);
GLRect GLUnionRect(GLRect aRect, GLRect bRect);
GLRect GLIntersectionRect(GLRect aRect, GLRect bRect);
GLRect GLOffsetRect(GLRect aRect, double dX, double dY);
void GLDivideRect(GLRect inRect, GLRect *slice, GLRect *rem, double amount, GLRectEdge edge);
void GLTorusPartition(GLRect world, GLRect view, GLRect *partition1, GLRect *partition2, GLRect *partition3, GLRect *partition4, BOOL wrapsHorizontally, BOOL wrapsVertically);

#pragma mark -
#pragma mark Range Comparison
#pragma mark

BOOL GLPointInRect(GLPoint aPoint, GLRect aRect);
BOOL GLMouseInRect(GLPoint aPoint, GLRect aRect, BOOL flipped);
BOOL GLContainsRect(GLRect aRect, GLRect bRect);
BOOL GLIntersectsRect(GLRect aRect, GLRect bRect);

#pragma mark -
#pragma mark String Representation
#pragma mark

NSString *NSStringFromGLPoint(GLPoint aPoint);
NSString *NSStringFromGLSize(GLSize aSize);
NSString *NSStringFromGLRect(GLRect aRect);

