/*
 *  GLGeometryTypes.h
 *  GLGeometry
 *
 *  Created by Jeffrey Early on 4/23/06.
 *  Copyright 2006, Early Innovation. All rights reserved.
 *
 */

#include <CoreServices/../Frameworks/CarbonCore.framework/Headers/MacTypes.h>

// from the "Description of WKBGeometry Representations"

/************************************************/
/*		Based on the "Description of			*/
/*		WKBGeometry Representations"			*/
/************************************************/

typedef enum {
	kGLPointType = 1,
    kGLLineStringType = 2,
    kGLPolygonType = 3,
    kGLMultiPointType = 4,
    kGLMultiLineStringType = 5,
    kGLMultiPolygonType = 6,
    kGLGeometryCollectionType = 7,

} GLGeometryType;

typedef struct GLPoint { 
	double x; 
	double y;
	int zone;
} GLPoint;

typedef struct GLPoint * __strong GLPointPtr;

typedef struct GLLinearRing  { 
	UInt32			numPoints; 
	GLPointPtr		points; 
} GLLinearRing;

typedef struct GLLinearRing * __strong GLLinearRingPtr;

typedef struct GLLineString { 
	UInt32			numPoints; 
	GLPointPtr		points;
} GLLineString;

typedef struct GLLineString * __strong GLLineStringPtr;

typedef struct GLPolygon { 
	UInt32			numRings; 
	GLLinearRingPtr rings; 
} GLPolygon;

typedef struct GLPolygon * __strong GLPolygonPtr;

typedef struct GLMultiPoint { 
	UInt32			numPoints; 
	GLPointPtr		points;
} GLMultiPoint;

typedef struct GLMultiPoint * __strong GLMultiPointPtr;

typedef struct GLMultiLineString { 
	UInt32			numLineStrings; 
	GLLineStringPtr	lineStrings; 
} GLMultiLineString;

typedef struct GLMultiLineString * __strong GLMultiLineStringPtr;

typedef struct GLMultiPolygon { 
	UInt32			numPolygons; 
	GLPolygonPtr	polygons; 
} GLMultiPolygon;

typedef struct GLMultiPolygon * __strong GLMultiPolygonPtr;

/************************************************/
/*		Based on NSGeometry						*/
/************************************************/

typedef struct GLSize {
    double width;		/* should never be negative */
    double height;		/* should never be negative */
} GLSize;

typedef struct GLRect {
    GLPoint origin;
    GLSize size;
} GLRect;

// These values are specifically chosen so that any two value '&' is zero.
// Extensively used by the clipping algorithms when generating region codes.
typedef enum GLRectEdge {
    GLBottomEdge = 1,
    GLTopEdge = 2,
	GLRightEdge = 4,
	GLLeftEdge = 8
} GLRectEdge;

/************************************************/
/*		Used by clipping algorithms				*/
/************************************************/

// These are essentially the region codes for the four corner regions.
typedef enum GLRectCorner {
    GLLowerRightCorner = 5,
    GLUpperRightCorner = 6,
	GLLowerLeftCorner = 9,
	GLUpperLeftCorner = 10
} GLRectCorner;

typedef struct {
	double a;
	double b;
	double c;
} GLProjectiveLine;