/*
 *  GLGeometryUtilities.h
 *  GLGeometry
 *
 *  Created by Jeffrey Early on 10/11/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#import <GLGeometryKit/GLGeometryTypes.h>
#import <GLGeometryKit/GLClippingRegion.h>

extern int edgeTableA[16];
extern int edgeTableB[16];
extern int maskTable[16];


// Computes the distances in meters between two WGS84 coordinates
double GLEarthDistance( GLPoint point1, GLPoint point2 );

// Given two points, this computes the line created by those two points in homogenous coordinates.
// a*x + b*y + c*w = 0.
GLProjectiveLine GLProjectiveLineFromPoints(GLPoint *pointA, GLPoint *pointB);

// Takes two projective lines (in homogenous coordinates) and returns a point in Euclidean coordinates
// where the two lines intersection. It *will* divide by zero if they don't intersect.
GLPoint GLPointIntersectionOfProjectiveLines( GLProjectiveLine lineA, GLProjectiveLine lineB );

// Cohen-Sutherland encoding of points.
// If you bitwise & two region codes together and they're not zero, the two points aren't in bounds.
int GLRegionCode(GLPoint * point, GLClippingRegion *clippingRegion);

// My fancy schmancy clipping algorithm for polygons, adapted from Skala's line clipping algorithm.
void GLClipLinearRing( GLLinearRingPtr inRing, GLLinearRingPtr outRing, GLClippingRegion *clippingRegion);

// This is exactly the Vaclav Skala's line clipping algoritm.
void GLClipLineString( GLLineStringPtr inLine, GLLineStringPtr outLine, GLClippingRegion *clippingRegion);

// Multipoint with points only in the clipping region.
void GLClipMultiPoint( GLMultiPointPtr inLine, GLMultiPointPtr outLine, GLClippingRegion *clippingRegion);

// Compute the centroid of a ring. The area is computed for free, so you can grab that as well.
GLPointPtr GLCentroid( GLLinearRingPtr ring, double *area );