//
//  GLMultiPolygonGeometry.h
//  GLGeometry
//
//  Created by Jeffrey Early on 4/23/06.
//  Copyright 2006, Early Innovation. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <GLGeometryKit/GLGeometry.h>

// A MultiPolygon is a two-dimensional geometric collection whose elements are Polygons. The interiors of 
// any two Polygons in a MultiPolygon may not intersect. The boundaries of any two Polygons in a 
// MultiPolygon may intersect at most at a finite number of points. 

@class GLClippingRegion;
@interface GLMultiPolygonGeometry : GLGeometry {

	GLMultiPolygonPtr aMultiPolygon;
	GLMultiPolygonPtr clippedMultiPolgon;
}

/************************************************/
/*		Object Initialization					*/
/************************************************/

#pragma mark -
#pragma mark Object Initialization
#pragma mark

// Initializes and returns a GLMultiPolygonGeometry instance with the wkb contents in the NSData object
- (id) initWithData: (NSData *) data;

- (id) initWithMultiPolygon: (GLMultiPolygon) multiPolygon;

- (id) initWithMultiPolygonPtr: (GLMultiPolygonPtr) multiPolygon;

/************************************************/
/*			Accessor Methods					*/
/************************************************/

#pragma mark -
#pragma mark Accessor Methods
#pragma mark

// Returns the structure value.
- (GLMultiPolygonPtr) multiPolygonGeometry;

- (GLMultiPolygonPtr) clippedMultiPolygonWithRegion: (GLClippingRegion *) clippingRegion;

@end
