//
//  GLPointGeometry.h
//  GLGeometry
//
//  Created by Jeffrey Early on 4/23/06.
//  Copyright 2006, Early Innovation. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <GLGeometryKit/GLGeometry.h>

/* GLPointGeometry */

// A Point is a 0-dimensional geometry and represents a single location in 
// coordinate space. A Point has a x- coordinate value and a y-coordinate value.

@class GLClippingRegion;
@interface GLPointGeometry : GLGeometry {

	GLPointPtr aPoint;
	GLPointPtr clippedPoint;
}

/************************************************/
/*		Object Initialization					*/
/************************************************/

#pragma mark -
#pragma mark Object Initialization
#pragma mark

// Initializes and returns a GLPointGeometry instance with the wkb contents in the NSData object
- (id) initWithData: (NSData *) data;

// initialize using an NSPoint (floats)
- (id) initWithNSPoint: (NSPoint) point;

// initialize using a GLPoint (doubles)
- (id) initWithPoint: (GLPoint) point;

// init with a GLPointPtr
- (id) initWithPointPtr: (GLPointPtr) point;

/************************************************/
/*			Accessor Methods					*/
/************************************************/

#pragma mark -
#pragma mark Accessor Methods
#pragma mark

// The x-coordinate value for this Point.
- (double) x;

// The y-coordinate value for this Point.
- (double) y;

// Returns the structure value.
- (GLPointPtr) pointGeometry;

// Returns cast as an NSPoint (floats).
- (NSPoint) asNSPoint;

- (GLPointPtr) clippedPointWithRegion: (GLClippingRegion *) clippingRegion;

@end
