/*
 *  GLSpatialObject.h
 *  GLGeometryKit
 *
 *  Created by Jeffrey Early on 9/30/06.
 *  Copyright 2006 Early Innovation. All rights reserved.
 *
 */

/* GLSpatialObject */

// This protocol will be attached to every object (feature) that is considered
// spatial. These objects can be dragged and dropped onto a user-defined
// theme, containing any other spatial objects.
//
// These objects should also register themselves as a particular drag type.

#import <GLGeometryKit/GLDrawContext.h>

@class GLGeometry, GLStyle;
@protocol GLSpatialObject

// returns the GLSpatialObjectController that owns the objects, so the plugin can be informed of changes
- (id) objectOwner;

// returns the name of the feature, will be displayed to the user
- (NSString *) featureName;

// returns a more detailed description of the feature, will be displayed to the user
- (NSString *) featureDescription;

// returns the kind of feature, will be displayed to the user
- (NSString *) kind;

// returns a geometry object, used for displaying
- (GLGeometry *) geometry;

// describes how the geometry should be drawn, including the label
- (GLStyle *) style;

// return a dictionary of NSPropertyDescription with key corresponding to the field name.
// These values are also likely to be directly implemented as methods in the
// object, "- (id) value". This will be used for exporting data and building NSPredicates.
- (NSDictionary *) propertiesByName;

@optional
// As a way to increase speed, a spatial object can optionally store geometries for a
// number of different resolutions.
- (GLGeometry *) geometryForResolution: (double) resolution;


@end

// *If* you really did define a spatialObject that conforms to the above protocol, then you can call
// this method with a valid drawContext and the object will go ahead and symbolize.
@class GLDrawContext, GLSpatialObject;
@interface NSObject (GLSpatialObjectDrawAdditions)

- (void) drawInContext: (NSObject <GLDrawContext> *) drawContext;

@end

