//
//  GLStyle.h
//  GLMapKit
//
//  Created by Jeffrey Early on 12/30/06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <GLGeometryKit/GLDrawContext.h>

@class GLGeometry, GLRule, GLSpatialObject;
@interface GLStyle : NSObject <NSCoding, NSCopying> {
	NSString *name;
	NSString *title;
	NSString *abstract;
	NSMutableArray *rules;
}

@property(copy) NSString *name, *title, *abstract;
@property(assign) NSMutableArray *rules;

// returns a the default style object for a given geometry type.
- (id) initForGeometry: (GLGeometry *) geometry;
+ (GLStyle *) styleForGeometry: (GLGeometry *) geometry;

// Tests for complete equality, all properties, all rules, and all symbolizers are equal
- (BOOL) isEqualToStyle: (GLStyle *) aStyle;

// Only requires that the two styles share the same rules, with the same type of symbolizers in the same order
- (BOOL) isStructurallyEqualToStyle: (GLStyle *) aStyle;

- (BOOL) addRule: (GLRule *) rule;
- (BOOL) removeRule: (GLRule *) rule;
- (BOOL) addRule: (GLRule *) rule atIndex: (NSUInteger) index;
- (BOOL) moveRuleFromIndex: (NSUInteger) oldIndex toIndex: (NSUInteger) newIndex;

/************************************************/
/*		Spatial Object Draw Additions			*/
/************************************************/

#pragma mark -
#pragma mark Spatial Object Draw Additions
#pragma mark

// Sorts through its rules and symbolizes the spatial object in the context.
- (void) drawSpatialObject: (GLSpatialObject *) spatialObject inContext: (NSObject <GLDrawContext> *) drawContext;
 
@end
